use clap::Parser;
use tool_helper::{Error, exit_with_error};
use std::{io::{BufRead, BufReader}, path::PathBuf, process::{Child, Command, Stdio}};

#[derive(Parser)]
#[clap(about = "Opens and scans a MAP file to print extended information", long_about = None)]
struct CommandLine {
    #[clap(value_parser, help="Input MAP file for scannning")]
    input:          PathBuf,
    #[clap(long="wsl", default_value_t=false)]
    use_wsl:        bool,
}

fn run_objdump(use_wsl: bool, input: PathBuf) -> Result<Child, Error> {
    let command_list = ["wsl", "objdump", "-x"];
    let start_idx    = {
        if use_wsl {0}
        else       {1}
    };
    let mut process = Command::new(command_list[start_idx]);

    for arg in command_list.iter().skip(start_idx + 1) {
        process.arg(arg);
    }
    process.arg(input.to_str().ok_or_else(||{Error::from_str("Failed converting input string")})?);

    Ok(process.stdout(Stdio::piped()).spawn()?)
}

fn run_main(cmd: CommandLine) -> Result<(), Error> {
    let mut child = run_objdump(cmd.use_wsl, cmd.input)?;
    if let Some(stdout) = &mut child.stdout {
        let mut line_iter = BufReader::new(stdout).lines().map(|l| l.unwrap()).into_iter();
        
        readmap::scan(|| {
            line_iter.next()
        })?;
        Ok(())
    }

    else {
        Err(Error::from_str("Failed opening \"stdout\" for objdump"))
    }
}

pub fn main() {
    match CommandLine::try_parse() {
        Ok(cmd_line) => {
            match run_main(cmd_line) {
                Ok(_)      => (),
                Err(error) => exit_with_error(error)
            }
        },
        Err(error) => {
            println!("{}", error)
        }
    }
}